var CeleryTask = Backbone.Model.extend({
    defaults: { // default attributes for new models
        'status': 'PENDING',
        'result': null
    },
    url : function() {
			return '/admin/async/task/' + this.get("id") + "/status/"
	},
    defaultOptions: { // default options, user can override on init
        interval: 5000, // poll interval (ms)
        autoPoll: true, // automatically poll the update URL
        urlRoot: '/admin/async/task/' // Get task status from /urlRoot/<task_id>
    },
    READY_STATES: ['SUCCESS', 'FAILURE', 'REVOKED'],
    initialize: function(model, options) {
        _.bindAll(this, 'poll', 'updateState', 'startPolling', 'stopPolling');

        // Set up options. User-specified options are checked first, followed
        // by the model defaults.
        // TODO add support for Collection-level defaults for new tasks
        this.opts = $.extend({}, this.defaultOptions, options);
        this.urlRoot = this.opts.urlRoot;

        // set up an event handler for status changes
        this.bind('change:status', this.updateState);

        // start the first poll, if required
        if (this.opts.autoPoll) {
            this.startPolling();
        }
    },
    parse: function(data) {
        // returns the object in the format expected by backbone
        return data.task;
    },
    poll: function() {
        // fetch the latest results from the server
        this.fetch({},{
        	 error: function(model, response) {
        	        model.set("staus","FAILURE")
        	    }
        });
    },
    startPolling: function() {
        this.timer = setInterval(this.poll, 5000);
    },
    stopPolling: function() {
        clearTimeout(this.timer);
    },
    updateState: function(model, resp) {
        // the status of the model has changed, so trigger the relevant
        // events
        newStatus = this.get('status');
        this.trigger('task_' + newStatus, this);
        if (this.collection) {
            this.collection.trigger('task_updated', this);
        }
        if (this.READY_STATES.indexOf(newStatus) != -1) {
            // stop polling if the task is complete
            this.stopPolling();
        }
    }

});

var CeleryTasks = Backbone.Collection.extend({
    model: CeleryTask,
    initialize: function(options) {
        _.bindAll(this, 'createTask', '_addTask', 'startPolling', 'stopPolling', 'pollAll',
                        '_pendingTasks');
    },
    _addTask: function(data) {
        // Parse the task_id from the response and add() the new task to the
        // collection
        this.add({id: data.task_id});
    },
    _pendingTasks: function() {
        // returns all tasks with status=='PENDING'
        return _.filter(this.models, function(item) {
            return item.get('status') == 'PENDING';
        })
    },
    createTask: function() {
        // Dummy function - creates a new task by GETting the add_task view
        $.getJSON('/tasks/add/', '', this._addTask);
    },
    pollAll: function() {
        _.each(this._pendingTasks(), function(task) {
            task.pol();
        });
    },
    startPolling: function() {
        _.each(this._pendingTasks(), function(task) {
            task.startPolling();
        });
    },
    stopPolling: function() {
        this.each(function(task) {
            task.stopPolling();
        });
    }
});

var TaskView = Backbone.View.extend({
    model: CeleryTask,
    tagName: "div",
    initialize: function () {
        this.listenTo(this.model, "change", this.render);
    },
    render: function () {
        result = this.model.get("result")
        status = this.model.get("status")
        var link,message,time;
        if (result){
        	 link = result.url
             message = result.message
             time = result.time_remaining
        }
       
        if (time) {
        	time = " - ETA " + (new Date).clearTime().addSeconds(time).toString('H:mm:ss')
        }
        else
        {
        	time = "" 
        }
        this.model.set("time", time)
        
        taskid = this.model.get("id")
        if(link){
	        myclass = $("[taskid=" + taskid + "]").attr("class")
	        $("[taskid=" + taskid + "]").replaceWith(function () {
				    return $('<a/>', {
				    	taskid: taskid,
				    	class: myclass,
				        html: this.innerHTML
				    });
	        })
	        $("[taskid=" + taskid + "]").attr("href", link)
        }
        	$("[taskid=" + taskid + "]").html(message)
        if (status == "SUCCESS") {
            p = 100
            el = this.$el
            id = this.model.get("id").toString()

            $("[taskid="+ this.model.get("id") +"]").show()
            setTimeout(function(){
                el.removeClass("bar-success").removeClass("bar-warning").removeClass("bar-info")
                el.addClass("bar-success")
            },500);

            $.ajax({
                url: '/admin/async/task/' + id + '/message/',
                type: 'GET',
                success: function(data) {
                    $("[taskid="+ id +"]").html(data.message);
                }
            });

        }else{
        	$("[taskid="+ this.model.get("id") +"]").hide()
        }
        if (status == "FAILURE") {
            p = 100
            el = this.$el
            
            setTimeout(function(){
            el.removeClass("bar-success").removeClass("bar-warning").removeClass("bar-info")
            el.addClass("bar-warning")
            },500);
        }
        if (status == "PROGRESS") {
        	this.$el.removeClass("bar-success").removeClass("bar-warning").removeClass("bar-danger")
            this.$el.addClass("bar-info")
            p = result.progress_percent
        }
        if (status == "PENDING") {
        	p = -1
        }
        mid = this.model.get("id")

    	this.$el.attr("aria-valuetransitiongoal", p).progressbar({
            display_text: 'center',
            status: status,
            mid: mid,
            percent_format: function(p) {
            	if (status=="PROGRESS"){
            		return p + " %" + get_time(mid)  
            	}else{
            		 return gettext(get_status(mid)) 
            	}
            	
            }
        });
   
   
        
    }

});

function get_time(id){
	tic = tasks.get({id: mid})
	if (!(tic)) return false
	return tic.get("time")
}
function get_status(id){
	tic = tasks.get({id: mid})
	if (!(tic)) return false
	return tic.get("status")
}


tasks = new CeleryTasks()

$(function() {
	$(".readybar").each(function(){
		status = $(this).attr("status")
		idtask = $(this).attr("id")
		if(status=='SUCCESS') {
			cl = "bar-success "
			$("[taskid="+ idtask +"]").show()
		}else{
			cl = "bar-danger"
			$("[taskid="+ idtask +"]").hide()
		}
		$(this).addClass(cl)
		text_status =  gettext(status)
		$(this).progressbar({
	        display_text: 'center',
	        text_status: text_status,
	        percent_format: function(p) { 
	        	return this.text_status
	        	}
	    });
	});
	$(".jmbcelerybar").each(function(){
		task_id = $(this).attr("id")
		
		var task = new CeleryTask({
		    id: task_id
		}, {
		    interval: 5000,
		    autoPoll: true
		});
		
		tasks.add(task)
		
		taskview = new TaskView({ model: task, el: "#" + task_id })
		taskview.render()
		
	})

});

