# -*- coding: utf-8 -*-
import json
from django.conf import settings
from django.utils.translation import ugettext as _
from django.contrib import admin
from django.conf.urls import patterns, url
from django.http import HttpResponse, Http404
try:
    from unittest.util import safe_repr
except ImportError:
    from django.utils.unittest.util import safe_repr


from celery.result import AsyncResult
from celery import states
from jmb.core.admin.options import ExtendibleModelAdmin
from jmb.async.models import Task

from jmb.async.tasks import AspettaTask


def JsonResponse(response):
    return HttpResponse(json.dumps(response), content_type='application/json')


class TaskAdmin(ExtendibleModelAdmin):  
    fieldsets = (
        (_('logs admin'), {
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (None, {
            'fields': ('task_id', 'app_label', 'title', 'status', 'message', 'url',)
        }),
    )
    list_display = (
        'creator', 'date_create', 'get_task_id', 'app_label', 'get_progress_bar', 'title',
        'get_message', 'get_edit_icon', 'get_delete_icon',
    )
    
    def dummy_task(modeladmin, request, queryset):
        result = AspettaTask().delay_or_fail('jmb.async')
    dummy_task.short_description = "Create test task"
    
    def get_task_id(self, obj):
        return '<a href="/admin/async/task/%s/status/">%s</a>' % (obj.task_id, obj.task_id)
    get_task_id.short_description = "task_id"
    get_task_id.allow_tags = True
    
    def get_message(self, obj):
        if obj.url:
            return '<a taskid="%s" href="%s%s">%s</a>' % (
                obj.task_id, settings.MEDIA_URL,
                obj.url, obj.message or 'Download')
        else:
            return '<div taskid="%s" >%s</div>' % (obj.task_id, obj.message or '')

    get_message.short_description = "message"
    get_message.allow_tags = True
    
    if settings.DEBUG:
        actions = [dummy_task]

    def get_urls(self):
        urls = super(TaskAdmin, self).get_urls()
        my_urls = patterns('',
            url(
                r'^(?P<task_id>[\w\d\-\.]+)/status/$',
                self.task_status,
                name='task_status'
            ),
            url(
                r'^(?P<task_id>[\w\d\-\.]+)/message/$',
                self.task_message,
                name='task_status'
            ),
        )
        return my_urls + urls


    def task_status(self, request, task_id):
        """Returns task status and result in JSON format."""
        result = AsyncResult(task_id)
        state, retval = result.state, result.result
        if isinstance(retval, dict) and 'url' in retval:
            retval['url'] = retval['url'][retval['url'].find(settings.MEDIA_URL):]
        response_data = dict(id=task_id, status=state, result=retval)
        if state in states.EXCEPTION_STATES:
            traceback = result.traceback
            response_data.update({
                'result': safe_repr(retval),
                'exc': "", # get_full_cls_name(retval.__class__)
                'traceback': traceback
            })
        return JsonResponse({'task': response_data})

    def task_message(self, request, task_id):
        """Returns task status and result in JSON format."""
        task = Task.objects.get(task_id=task_id)
        response_data = dict(id=task_id, message=task.message)
        return JsonResponse(response_data)

 
admin.site.register(Task, TaskAdmin)
